;;;; -*- Mode: LISP; Syntax: Common-Lisp; Base: 10;                             -*-
;;;; ------------------------------------------------------------------------------
;;;; File name: setrule.lsp
;;;;    System: LTRE
;;;;   Version: 4.1
;;;;    Author: Kenneth D. Forbus, Johan de Kleer
;;;;   Created: Feb 7, 1996
;;;;  Modified: everett on Sat Jul 6 20:45:57 1996
;;;;   Purpose: Rule for enforcing constraints on sets
;;;; Copyright: 1986, 1989, 1990, 1991, 1995 Kenneth D. Forbus, 
;;;;            Northwestern University, and Johan de Kleer, Xerox Corporation.
;;;;            All Rights Reserved.
;;;; -----------------------------------------------------------------------------

(in-package :LTRE)

(LTRE:RULE-FILE "SRGC")

(rule ((:TRUE (cl-user::set ?name) :VAR ?f1))
  (rule ((:INTERN (?name cl-user::members ?construal1) :VAR ?f2))
    (rule ((:INTERN (?name cl-user::has-member ?new) :VAR ?f3
		    :TEST (not (member ?new ?construal1
				       :TEST #'equal))))
	  (rassert! (:IMPLIES (:AND ?f1 ?f2) (:NOT ?f3))
		    :NOT-IN-SET))
    (rule ((:INTERN (?name cl-user::MEMBERS ?construal2) :VAR ?f3))
	  (when (and (form< ?f2 ?f3)
		     (set-exclusive-or ?construal1
				       ?construal2
				       :TEST #'equal))
		(rassert! (:NOT (:AND ?f1 ?f2 ?f3))
			  :CONSTRUAL-UNIQUENESS)))))

;;; It's important to avoid duplicate justifications, especially in
;;; large problems.  Do this by reifying justifications made by the
;;; CWA code in the LTRE database, using this rule to translate the
;;; statement into the appropriate clauses.  (These statements can
;;; safely be premises, because they are inviolate: only the
;;; statements which participate in them can be wrong.

(rule ((:INTERN (CWA-JUSTIFICATION ?ante ?conse) :VAR ?cwaj))
  (rassert! (:IMPLIES ?cwaj
		      (:IMPLIES ?ante ?conse))
	    :CWA-JUSTIFICATION))
;;; ----------------------------------------------------------------------------
;;; END OF CODE